
%   Written by Wesley Zun ( November 2005 )

close all;
clear all;
clc;

echo on;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       SSFP DEMO
%
%   Move the figure so that you can see it with command window,
%   and do NOT close figures during demo. They will close 
%   automatically when you press any key to continue.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

echo off;
disp('Press any key to continue');
pause;
clc;

echo on;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   These are balance-SSFP profiles in the steady state according 
%   to different flip angles 10, 30, 60, 120, and 160 degree.
%   Different flip angle produces a different magnitude profile
%   while it has the same phase plot.
%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

flip= [10 30 60 120 160]*pi/180;
T1=1000;
T2=300;
TR=8;
TE=TR/2;
df=-250:250;
phi=0;
echo off;

for n=1:length(flip)
    for k=1:length(df)
        M=ssfp(flip(n),T1,T2,TE,TR,df(k),phi);
        Mxy=M(1)+i*M(2);
        Mamp(k,n)=abs(Mxy);
        Mang(k,n)=angle(Mxy);
    end
end

disp('  ');
disp('flip angle = 10 degree');

subplot(2,1,1);
plot(df,Mamp(:,1),'b');
title('flip angle = 10 degree');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;

disp('  ');
disp('flip angle = 30 degree');

subplot(2,1,1);
plot(df,Mamp(:,1),'k',df,Mamp(:,2),'b');
title('flip angle = 30 degree');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'k',df,180/pi*Mang(:,2),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;

disp('  ');
disp('flip angle = 60 degree');

subplot(2,1,1);
plot(df,Mamp(:,1),'k',df,Mamp(:,2),'k',df,Mamp(:,3),'b');
title('flip angle = 60 degree');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'k',df,180/pi*Mang(:,2),'k',df,180/pi*Mang(:,3),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;

disp('  ');
disp('flip angle = 120 degree');

subplot(2,1,1);
plot(df,Mamp(:,1),'k',df,Mamp(:,2),'k',df,Mamp(:,3),'k',df,Mamp(:,4),'b');
title('flip angle = 120 degree');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'k',df,180/pi*Mang(:,2),'k',df,180/pi*Mang(:,3),'k',df,180/pi*Mang(:,4),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;

disp('  ');
disp('flip angle = 160 degree');

subplot(2,1,1);
plot(df,Mamp(:,1),'k',df,Mamp(:,2),'k',df,Mamp(:,3),'k',df,Mamp(:,4),'k',df,Mamp(:,5),'b');
title('flip angle = 160 degree');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'k',df,180/pi*Mang(:,2),'k',df,180/pi*Mang(:,3),'k',df,180/pi*Mang(:,4),'k',df,180/pi*Mang(:,5),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;
clear all;
clf reset;
clc;

echo on;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Here are SSFP profiles for different phase shift between
%   each RF excitation. This demontrates profile is shifted by
%   RF phase shift.
%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

flip= 60 *pi/180;
T1=1000;
T2=300;
TR=8;
TE=TR/2;
df=-250:250;
phi=[0 -pi/2 -pi -3/2*pi] ;
echo off;

for n=1:length(phi)
    for k=1:length(df)
        M=ssfp(flip,T1,T2,TE,TR,df(k),phi(n));
        Mxy=M(1)+i*M(2);
        Mamp(k,n)=abs(Mxy);
        Mang(k,n)=angle(Mxy);
    end
end

disp('  ');
disp('RF phase shift = 0 rad');

subplot(2,1,1);
plot(df,Mamp(:,1),'b');
title('RF phase shift = 0 rad');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;

disp('  ');
disp('RF phase shift = pi/2 rad');

subplot(2,1,1);
plot(df,Mamp(:,1),'k',df,Mamp(:,2),'b');
title('RF phase shift = \pi/2 rad');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'k',df,180/pi*Mang(:,2),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;

disp('  ');
disp('RF phase shift = pi rad');

subplot(2,1,1);
plot(df,Mamp(:,1),'k',df,Mamp(:,2),'k',df,Mamp(:,3),'b');
title('RF phase shift = \pi rad');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'k',df,180/pi*Mang(:,2),'k',df,180/pi*Mang(:,3),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;

disp('  ');
disp('flip angle = 3*pi/2 rad');

subplot(2,1,1);
plot(df,Mamp(:,1),'k',df,Mamp(:,2),'k',df,Mamp(:,3),'k',df,Mamp(:,4),'b');
title('RF phase shift = 3*\pi/2 rad');
xlabel('frequency (Hz)');
ylabel('magnitude');
grid on;

subplot(2,1,2);
plot(df,180/pi*Mang(:,1),'k',df,180/pi*Mang(:,2),'k',df,180/pi*Mang(:,3),'k',df,180/pi*Mang(:,4),'b');
xlabel('frequency (Hz)');
ylabel('phase (degree)');
grid on;

disp('Press any key to continue');
pause;
clear all;
clf reset;
clc;

echo on;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   This shows steady state magnetization over the course of a TR.
%   3D visualization is on the left and Mxy in the transverse
%   plane is on the right for 3 different off-resonance frequencies
%   10 Hz, 25 Hz, and 40 Hz.
%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
%   Non-alternating SSFP
%

flip= 45 *pi/180;
T1=1000;
T2=300;
TR=8;
TE=TR/2;
df=[10 25 40];
phi=0;
echo off;

for n=1:length(df)
    
    P(:,1,n)=ssfp_M1(flip,T1,T2,TE,TR,df(n),phi);
    P(:,2,n)=yrot(flip)*P(:,1,n);

    Q(:,1,n)=P(:,2,n);
    num=50;
    [At,Bt] = freeprecess(TR/(num),T1,T2,df(n));
    for k=1:num
        Q(:,k+1,n)=At*Q(:,k,n)+Bt;
    end
    
    Z(:,:,n)=[P(:,:,n) Q(:,:,n)];

end

for i=1:2
    for k=1:length(Z)
        
        subplot(1,2,1);
        plot3(Z(1,:,1),Z(2,:,1),Z(3,:,1),'k',...
            Z(1,:,2),Z(2,:,2),Z(3,:,2),'k',...
            Z(1,:,3),Z(2,:,3),Z(3,:,3),'k',...
            Z(1,k,1),Z(2,k,1),Z(3,k,1),'ob',...
            Z(1,k,2),Z(2,k,2),Z(3,k,2),'og',...
            Z(1,k,3),Z(2,k,3),Z(3,k,3),'or',...
            [0 Z(1,k,1)],[0 Z(2,k,1)],[0 Z(3,k,1)],'b',...
            [0 Z(1,k,2)],[0 Z(2,k,2)],[0 Z(3,k,2)],'g',...
            [0 Z(1,k,3)],[0 Z(2,k,3)],[0 Z(3,k,3)],'r'...
        );
        xlabel('x');
        ylabel('y');
        zlabel('z');
        
        axis equal;
        axis([-0.3 0.3 -0.3 0.3 0 0.6]);
        grid on;
        
        subplot(1,2,2);
        plot(Z(1,:,1),Z(2,:,1),'k',...
            Z(1,:,2),Z(2,:,2),'k',...
            Z(1,:,3),Z(2,:,3),'k',...
            Z(1,k,1),Z(2,k,1),'ob',...
            Z(1,k,2),Z(2,k,2),'og',...
            Z(1,k,3),Z(2,k,3),'or',...
            [0 Z(1,k,1)],[0 Z(2,k,1)],'b',...
            [0 Z(1,k,2)],[0 Z(2,k,2)],'g',...
            [0 Z(1,k,3)],[0 Z(2,k,3)],'r'...
        );
        
        xlabel('x');
        ylabel('y');
        axis equal;
        axis([-0.3 0.3 -0.3 0.3]);
        grid on;
        
        drawnow;
    end
end


disp('Press any key to continue');
pause;
clear all;
clf reset;

echo on;

%
%   Alternating SSFP.
%

flip= 45 *pi/180;
T1=1000;
T2=300;
TR=8;
TE=TR/2;
df=[10 25 40];
phi=pi;
echo off;

clf reset;

for n=1:length(df)
    
    P(:,1,n)=ssfp_M1(flip,T1,T2,TE,TR,df(n),phi);
    P(:,2,n)=yrot(flip)*P(:,1,n);

    Q(:,1,n)=P(:,2,n);
    num=50;
    [At,Bt] = freeprecess(TR/(num),T1,T2,df(n));
    for k=1:num
        Q(:,k+1,n)=At*Q(:,k,n)+Bt;
    end

    R(:,1,n)=Q(:,num+1,n);
    R(:,2,n)=yrot(-flip)*R(:,1,n);

    S(:,1,n)=R(:,2,n);
    for k=1:num
        S(:,k+1,n)=At*S(:,k,n)+Bt;
    end

    Z(:,:,n)=[P(:,:,n) Q(:,:,n) R(:,:,n) S(:,:,n)];

end


for i=1:2
    for k=1:length(Z)
        
        subplot(1,2,1);
        plot3(Z(1,:,1),Z(2,:,1),Z(3,:,1),'k',...
            Z(1,:,2),Z(2,:,2),Z(3,:,2),'k',...
            Z(1,:,3),Z(2,:,3),Z(3,:,3),'k',...
            Z(1,k,1),Z(2,k,1),Z(3,k,1),'ob',...
            Z(1,k,2),Z(2,k,2),Z(3,k,2),'og',...
            Z(1,k,3),Z(2,k,3),Z(3,k,3),'or',...
            [0 Z(1,k,1)],[0 Z(2,k,1)],[0 Z(3,k,1)],'b',...
            [0 Z(1,k,2)],[0 Z(2,k,2)],[0 Z(3,k,2)],'g',...
            [0 Z(1,k,3)],[0 Z(2,k,3)],[0 Z(3,k,3)],'r'...
        );
        xlabel('x');
        ylabel('y');
        zlabel('z');
        
        axis equal;
        axis([-0.3 0.3 -0.3 0.3 0 0.6]);
        grid on;
        
        subplot(1,2,2);
        plot(Z(1,:,1),Z(2,:,1),'k',...
            Z(1,:,2),Z(2,:,2),'k',...
            Z(1,:,3),Z(2,:,3),'k',...
            Z(1,k,1),Z(2,k,1),'ob',...
            Z(1,k,2),Z(2,k,2),'og',...
            Z(1,k,3),Z(2,k,3),'or',...
            [0 Z(1,k,1)],[0 Z(2,k,1)],'b',...
            [0 Z(1,k,2)],[0 Z(2,k,2)],'g',...
            [0 Z(1,k,3)],[0 Z(2,k,3)],'r'...
        );
        xlabel('x');
        ylabel('y');
        axis equal;
        axis([-0.3 0.3 -0.3 0.3]);
        grid on;
        
        drawnow;
    end
end


disp('Press any key to continue');
pause;
clear all;
clf reset;
clc;

echo on;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   This illustrates the transient approach to steady state
%   of the vector M at TE for the first 70 TRs.
%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

flip= 45 *pi/180;
T1=300;
T2=100;
TR=8;
TE=TR/2;
df=[0 90 120]/360*125;
phi=0;
echo off;

for n=1:length(df)
    
    clf reset;
        
    [Atr,Btr] = freeprecess(TR-TE,T1,T2,df(n));
    [Ate,Bte] = freeprecess(TE,T1,T2,df(n));

    M(:,1)=[0;0;1];
    for k=1:70
        M(:,k+1)=Atr*yrot(flip)*(Ate*M(:,k)+Bte)+Btr;
    end
    
    if n==1
        disp('  ');
        disp('off-resonance = 0 rad for a TR');
    elseif n==2
        disp('  ');
        disp('off-resonance = pi rad for a TR');
    else
        disp('  ');
        disp('off-resonance = 2/3*pi rad for a TR');
    end
   
    for k=2:length(M)
        
        subplot(1,2,1);
        plot3([M(1,k-1) M(1,k)],[M(2,k-1) M(2,k)],[M(3,k-1) M(3,k)]);
        xlabel('x');
        ylabel('y');
        zlabel('z');
        axis equal;
        axis([-1 1 -1 1 -1 1]);
        grid on;
        hold on;
    
        subplot(1,2,2);
        plot([M(1,k-1) M(1,k)],[M(2,k-1) M(2,k)]);
        xlabel('x');
        ylabel('y');
        axis equal;
        axis([-1 1 -1 1]);
        grid on;
        hold on;
    
        drawnow;   
    end
        
    disp('Press any key to continue');
    pause;

end
clc;

disp('                      ');
disp('          THE END     ');
disp('                      ');


